use bitvec::{order::Msb0, view::BitView};

const WIDTH: i32 = 96;
const HEIGHT: i32 = 38;

#[no_mangle]
pub extern "C" fn next_frame(frame: u16, ptr: *mut u8) -> u8 {
    let out = unsafe { std::slice::from_raw_parts_mut(ptr, 456) }.view_bits_mut::<Msb0>();

    if frame > 100 {
        return 1;
    }

    let cat = include_bytes!(concat!(env!("OUT_DIR"), "/cat.bin")).view_bits::<Msb0>();
    let meow = include_bytes!(concat!(env!("OUT_DIR"), "/meow.bin")).view_bits::<Msb0>();

    let thres = WIDTH / 4;

    let dx = (frame as i32 * 4 - thres).min(thres);

    for x in 0..96 {
        for y in 0..38 {
            let out_idx = index(x, y);
            let in_idx = index(x - dx, y);

            out.set(out_idx, cat[in_idx]);
        }
    }

    if dx == thres && frame / 7 % 2 == 0 {
        // make meow flash on and off every 4 frames for the remaining time

        for x in 0..96 {
            for y in 0..38 {
                let idx = index(x, y);

                out.set(idx, meow[idx] || out[idx]);
            }
        }
    }

    0
}

fn index(x: i32, y: i32) -> usize {
    if x < 0 || y < 0 || x >= WIDTH || y >= HEIGHT {
        return 0;
    }

    x as usize + y as usize * WIDTH as usize
}
